@@ -1,19 +1,62 @@
 # -*- coding: utf-8 -*-
+"""
+This code is taken from the itertree package:
+https://pypi.org/project/itertree/
+GIT Home:
+https://github.com/BR1py/itertree
+The documentation can be found here:
+https://itertree.readthedocs.io/en/latest/index.html
+
+The code is published under MIT license:
+
+The MIT License (MIT)
+Copyright © 2022 <copyright holders>
+
+Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
+documentation files (the “Software”), to deal in the Software without restriction, including without limitation
+the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and
+to permit persons to whom the Software is furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in all copies or substantial
+portions of the Software.
+
+THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO
+THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
+OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
+OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
+For more information see: https://en.wikipedia.org/wiki/MIT_License
+
+
+This part of code contains the standard iTree serializers (JSON and rendering)
+"""
+
 from __future__ import absolute_import
 
 import os
+import platform
 import gzip
 import hashlib
 from collections import OrderedDict
 
 # For serializing we (try) to import some modules:
+
 IS_ORJSON=False
+DECODE = False
+
 try:
     import orjson as JSON
     IS_ORJSON=True
     # orjson is much faster then standard json but might not be available
 except:
     import json as JSON
+
+    test=bytes(JSON.dumps({'test':'123','HASH':hashlib.sha1(b'asjhdahsdh').hexdigest()}).encode('utf8', errors='backslashreplace'))
+    try:
+        JSON.loads(test)
+    except TypeError:
+        DECODE=True
+
 try:
     import numpy as np
     # only needed in case of numpy arrays in data
@@ -25,9 +68,9 @@ from .itree_helpers import *
 from .itree_filter import iTFilterItemType
 
 from .itree_data import iTData, iTDataReadOnly
-from .itree_main import iTree, iTreeReadOnly, iTreeLink, iTreeTemporary
+from .itree_main import iTree, iTreeReadOnly, iTreeLink, iTreeTemporary,iTreePlaceHolder
 
-DT_SERIALIZE_VERSION = "1.1.0"
+DT_SERIALIZE_VERSION = "1.1.1"
 
 
 class iTStdObjSerializer(object):
@@ -41,9 +84,15 @@ class iTStdObjSerializer(object):
     DATA = 'DT'
     LINK = 'LK'
     TAG = 'TG'
+    IDX = 'IDX'
+
     DATA_MODELL = 'DM'
     DTYPE = 'TP'
     DATA_CONTAINER = 'DC'
+    ITREE_ITEMS_DECODE={'iT':iTree,'iTRO':iTreeReadOnly,'iTl':iTreeLink,'iTPH':iTreePlaceHolder,'iTI':TagIdx}
+    ITREE_ITEMS_ENCODE = {iTree:'iT', iTreeReadOnly:'iTRO', iTreeLink:'iTl' , iTreePlaceHolder:'iTPH',TagIdx:'iTI' }
+    OTHER_ITEMS_DECODE = {'d':dict ,'D': iTData , 'DR':iTDataReadOnly , 'OD':OrderedDict }
+    OTHER_ITEMS_ENCODE={dict:'d',iTData:'D',iTDataReadOnly:'DR',OrderedDict:'OD'}
 
     def encode(self, o):
         """
@@ -53,34 +102,27 @@ class iTStdObjSerializer(object):
         """
         encode = self.encode
         t = type(o)
-        dtype = None
-        if t is iTree:
-            dtype = 'iT'
-        elif t is iTreeReadOnly:
-            dtype = 'iTRO'
-        elif t is iTreeLink:
-            dtype = 'iTl'
+        dtype = self.ITREE_ITEMS_ENCODE.get(t)
         if dtype is not None:
-            dt_dict = {self.DTYPE: dtype, self.TAG: encode(o._tag)}
-            if t is iTreeLink:
-                if o._link is not None:
-                    dt_dict[self.LINK] = (o._link.file_path, o._link.key_path)
-            data = o._data
-            if not data.is_empty:
-                dt_dict[self.DATA] = encode(data)
-            dt_dict[self.TREE] = [encode(item) for item in
-                                  o.iter_children(item_filter=iTFilterItemType(iTreeTemporary, invert=True))]
+            if t is TagIdx:
+                return {self.DTYPE: dtype, self.TAG: encode(o.tag),self.IDX:o.idx}
+            dt_dict = {self.DTYPE: dtype, self.TAG: encode(o.tag)}
+            if t is not iTreePlaceHolder:
+                data = o._data
+                if not data.is_empty:
+                    dt_dict[self.DATA] = encode(data)
+                if t is iTreeLink:
+                    if o._link is not None:
+                        dt_dict[self.LINK] = (o._link.file_path, encode(o._link.key_path))
+                    result_list=[encode(item) for item in o.iter_locals(add_placeholders=True)]
+                else:
+                    result_list =[encode(item) for item in
+                                      o.iter_children(item_filter=iTFilterItemType(iTreeTemporary, invert=True))]
+                dt_dict[self.TREE] = result_list
             return dt_dict
         if (t is str) or (t is int) or (t is float) or (t is bool):
             return o
-        if t is dict:
-            dtype = 'd'
-        elif t is iTData:
-            dtype = 'D'
-        elif t is iTDataReadOnly:
-            dtype = 'DR'
-        elif t is OrderedDict:
-            dtype = 'OD'
+        dtype=self.OTHER_ITEMS_ENCODE.get(t)
         if dtype is not None:
             return {self.DTYPE: dtype,
                     self.DATA_CONTAINER: [(encode(k), encode(v)) for k, v in o.items()]}
@@ -93,7 +135,7 @@ class iTStdObjSerializer(object):
         except:
             return o
 
-    def decode(self, raw_o, load_links=True):
+    def decode(self, raw_o):
         """
         decode the given raw_object back to the original object
         :param raw_o: raw_object (dict or list)
@@ -106,22 +148,29 @@ class iTStdObjSerializer(object):
             o_type = raw_o.get(self.DTYPE)
             if o_type is None:
                 return {decode(key): decode(v) for key, v in raw_o}
-            elif o_type == 'iT' or o_type == 'iTL' or o_type == 'iTRO':
+            elif o_type in self.ITREE_ITEMS_DECODE:
                 tag = decode(raw_o.get(self.TAG))
-                data = raw_o.get(self.DATA)
-                if data is not None:
-                    data = decode(data)
-                link = raw_o.get(self.LINK)
-                if link is not None:
-                    new_item = iTreeLink(tag, data=data, link_file_path=link[0], link_key_path=link[1])
-                    if load_links:
-                        new_item.load_links()
+                if o_type == 'iTPH':
+                    return iTreePlaceHolder(tag)
+                elif o_type == 'iTI':
+                    return TagIdx(tag,raw_o.get(self.IDX))
                 else:
+                    data = raw_o.get(self.DATA)
+                    if data is not None:
+                        data = decode(data)
                     tree = raw_o.get(self.TREE)
-                    if o_type == 'iT':
-                        new_item = iTree(tag, data=data, subtree=[decode(i) for i in tree])
+                    link = raw_o.get(self.LINK)
+                    if link is not None:
+                        new_item = iTreeLink(tag, data=data,
+                                             link_file_path=link[0],
+                                             link_key_path=decode(link[1]),
+                                             load_links=False,
+                                             subtree=[decode(i) for i in tree])
                     else:
-                        new_item = iTreeReadOnly(tag, data=data, subtree=[decode(i) for i in tree])
+                        if o_type == 'iT':
+                            new_item = iTree(tag, data=data, subtree=[decode(i) for i in tree])
+                        else:
+                            new_item = iTreeReadOnly(tag, data=data, subtree=[decode(i) for i in tree])
                 return new_item
             elif o_type == 'OD':
                 return OrderedDict([(decode(i[0]), decode(i[1])) for i in raw_o[self.DATA_CONTAINER]])
@@ -241,7 +290,7 @@ class iTStdJSONSerializer(object):
             with open(file_path, 'wb') as fh:
                 fh.write(data)
 
-    def loads(self, source_str, check_hash=True, load_links=True):
+    def loads(self, source_str, check_hash=True, load_links=True,_source=None):
         """
         create an iTree object by loading from a string.
 
@@ -249,18 +298,36 @@ class iTStdJSONSerializer(object):
         :param check_hash: True the hash of the file will be checked and the loading will be stopped if it doesn't match
                            False - do not check the iTree hash
         :param load_links: True - linked iTree objects will be loaded
+        :param _source: Path of a loaded source file (for internal use)
+
         :return: iTree object loaded from file
         """
         header_str, dt_str = source_str.split(b'}***DT***')
-        header_dict = JSON.loads(header_str + b'}')
+        if DECODE:
+            header_dict = JSON.loads((header_str + b'}').decode('utf-8'))
+        else:
+            header_dict = JSON.loads(header_str + b'}')
         if header_dict['VERSION'] != DT_SERIALIZE_VERSION:
             raise PermissionError('Wrong version of DataTree data please convert first!')
         if check_hash and ('HASH' in header_dict):
             a = hashlib.sha1(dt_str).hexdigest()
             if hashlib.sha1(dt_str).hexdigest() != header_dict['HASH']:
                 raise PermissionError('Given DataTree data is corrupted (wrong hash)!')
-        raw_o = JSON.loads(dt_str)
-        return self.obj_serializer.decode(raw_o, load_links=load_links)
+        if DECODE:
+            raw_o = JSON.loads(dt_str.decode('utf-8'))
+        else:
+            raw_o = JSON.loads(dt_str)
+        new_tree=self.obj_serializer.decode(raw_o)
+        if _source is not None:
+            if new_tree._link is None:
+                link_obj=iTLink()
+                link_obj._source_path=_source
+                new_tree._link = link_obj
+            else:
+                new_tree._link._source_path=_source
+        if load_links:
+            new_tree.load_links()
+        return new_tree
 
     def load(self, file_path, check_hash=True, load_links=True):
         """
@@ -286,9 +353,11 @@ class iTStdJSONSerializer(object):
             # we might have an already unzipped file!
             pass
         # data=data.decode(decode)
-        return self.loads(data, check_hash=check_hash, load_links=load_links)
-
-
+        try:
+            file_path=os.path.abspath(file_path)
+        except:
+            pass
+        return self.loads(data, check_hash=check_hash, load_links=load_links,_source=file_path)
 
 
 class iTStdRenderer(object):
@@ -310,23 +379,27 @@ class iTStdRenderer(object):
         :return: string containing the item information
         """
         t = type(item)
-        link_str = ''
-        class_name = 'iTree'
+        link_str = u''
+        class_name = u'iTree'
         if t is iTreeReadOnly:
-            class_name = 'iTreeReadOnly'
+            class_name = u'iTreeReadOnly'
         elif t is iTreeLink:
-            class_name = 'iTreeLink'
+            class_name = u'iTreeLink'
             if item._link is not None:
-                link_str = ', link=%s' % repr(item._link)
+                link_str = u', link=%s' % repr(item._link)
+        elif t is iTreePlaceHolder:
+            class_name = u'iTreePlaceHolder'
         elif t is iTreeTemporary:
-            class_name = 'iTreeTemporary'
+            class_name = u'iTreeTemporary'
         data = item._data
+        out=u''
         if data.is_empty:
-            return '%s(%s%s)' % (class_name, repr(item.tag), link_str)
+            out= u'%s(%s%s)' % (class_name, repr(item.tag), link_str)
         if data.is_no_key_only:
-            return '%s(%s%s, data=%s)' % (class_name, repr(item.tag), link_str, repr(item.d_get()))
+            out= u'%s(%s%s, data=%s)' % (class_name, repr(item.tag), link_str, repr(item.d_get()))
         else:
-            return '%s(%s%s, data=%s)' % (class_name, repr(item.tag), link_str, repr(item._data))
+            out= u'%s(%s%s, data=%s)' % (class_name, repr(item.tag), link_str, repr(item._data))
+        return out
 
     def render2(self, itree_object, item_filter=None, _level=0):
         """
@@ -337,7 +410,7 @@ class iTStdRenderer(object):
         :param _level: internal parameter for recursive calls (do not use)
         :return: string containing the pretty print aoutput
         """
-        if not hasattr(itree_object, 'is_iTree'):
+        if not isinstance(itree_object, iTree):
             raise TypeError('Can only render iTree objects, got %s instead' % (type(itree_object)))
         if _level == 0:
             print(self.__create_item_string(itree_object))
@@ -348,7 +421,7 @@ class iTStdRenderer(object):
         if type(itree_object) is iTreeLink:
             header = self._link_heading
         for item in itree_object.iter_children(item_filter=item_filter):
-            print(''.join([' ' * (self._identation * _level), header, self.__create_item_string(item)]))
+            print(u''.join([u' ' * (self._identation * _level), header, self.__create_item_string(item)]))
             self.render(item, item_filter=item_filter, _level=_level + 1)
 
     def renders2(self, itree_object, item_filter=None, _level=0):
@@ -361,24 +434,24 @@ class iTStdRenderer(object):
         :param _level: internal parameter for recursive calls (do not use)
         :return: string containing the pretty print aoutput
         """
-        if not hasattr(itree_object, 'is_iTree'):
+        if not isinstance(itree_object, iTree):
             raise TypeError('Can only render iTree objects, got %s instead' % (type(itree_object)))
         if _level == 0:
             output = self.__create_item_string(itree_object)
         else:
-            output = ''
+            output = u''
         if (item_filter is not None) and (not item_filter(itree_object)):
             return ''
         sub_tree = None
         for item in itree_object.iter_children(item_filter=item_filter):
-            output = ''.join(
-                [output, '\n', ' ' * (self._identation * _level), self._heading, self.__create_item_string(item),
+            output = u''.join(
+                [output, u'\n', u' ' * (self._identation * _level), self._heading, self.__create_item_string(item),
                  self.renders(item, item_filter=item_filter, _level=_level + 1)])
         return output
 
     def render(self, itree_object, item_filter=None):
         """
-         creates a pretty print from iTree object
+         creates a pretty print from iTree object and prints it stdout
 
          Note:: Filtered renderings contains always the root object and the added children might have
                confusing indentation levels because the parent elements might be filtered out
@@ -414,17 +487,21 @@ class iTStdRenderer(object):
         :param _only_print_tree: True/False print or create return string
         :return: string containing the pretty print a output
         """
-        if not hasattr(itree_object, 'is_iTree'):
+        if not isinstance(itree_object, iTree):
             raise TypeError('Can only render iTree objects, got %s instead' % (type(itree_object)))
         output=[]
         items=[itree_object]
         if item_filter is not None:
             if _only_print_tree:
-                print(''.join([self.__create_item_string(itree_object)]))
+                out=u''.join([self.__create_item_string(itree_object)])
+                try:
+                    print(out)
+                except UnicodeEncodeError:
+                    print(out.encode().decode('cp1252'))
             else:
                 output.append(
-                    ''.join([self.__create_item_string(itree_object), '\n']))
-        heading = ''
+                    u''.join([self.__create_item_string(itree_object), '\n']))
+        heading = u''
         while 1:
             if len(items)==0:
                 break
@@ -445,9 +522,13 @@ class iTStdRenderer(object):
                 continue
             if item_filter is None or item_filter(item):
                 if _only_print_tree:
-                    print(''.join([' ' * (self._identation * level), heading, self.__create_item_string(item)]))
+                    out=u''.join([u' ' * (self._identation * level), heading, self.__create_item_string(item)])
+                    try:
+                        print(out)
+                    except UnicodeEncodeError:
+                        print(out.encode().decode('cp1252'))
                 else:
-                    output.append(''.join([' ' * (self._identation * level), heading, self.__create_item_string(item),'\n']))
+                    output.append(u''.join([u' ' * (self._identation * level), heading, self.__create_item_string(item),'\n']))
             if level==0:
                 heading = self._heading
             new_items=list(item.iter_children())
@@ -456,5 +537,5 @@ class iTStdRenderer(object):
             else:
                 # replace the original item with the children
                 parent_list[0]=new_items
-        return ''.join(output)
+        return u''.join(output)
 
